#!/usr/bin/python3
import os
import sys
import argparse
import configparser

from os.path import abspath, join, exists

from verbump import git, constants, version, __version__


def main():
    parser = argparse.ArgumentParser(
        description="Easy version incrementing CLI tool",
    )
    parser.add_argument(
        "part",
        type=str,
        default=constants.V_PATCH,
        nargs="?",
        help="Version part",
        choices=constants.version_parts,
    )
    parser.add_argument(
        "--version",
        action="version",
        version="verbump %s" % __version__,
    )
    parser.add_argument(
        "--config",
        "-c",
        default=join(os.getcwd(), ".verbump.ini"),
        help="Set config file",
    )
    args = parser.parse_args()

    config = configparser.RawConfigParser(
        defaults=dict(
            commit="false",
            tag="false",
            commit_format=constants.commit_format,
            tag_format=constants.tag_format,
        )
    )
    config.read(args.config)

    part = constants.version_parts.index(args.part)
    committed = False
    tagged = False
    for section in config.sections():
        verpath = abspath(section.split(":")[1])
        pattern = config.get(section, "pattern")

        if not exists(verpath):
            sys.exit("Cannot find version file '%s'" % verpath)

        # bump
        with open(verpath) as f:
            content, oldver, newver = version.bump(f.read(), pattern, part)

        print("Version bump from %s to %s on '%s'" % (oldver, newver, verpath))

        with open(verpath, "w") as f:
            f.write(content)

        # git commit
        need_commit = config.get(section, "commit")
        if not committed and need_commit and need_commit.lower() == "true":
            if git.has_stage():
                sys.exit("Stage must be empty")
            git.add(verpath)
            git.commit(config.get(section, "commit_format") % newver)
            print("Committed %s" % newver)
            committed = True

        # git tag
        need_tag = config.get(section, "tag")
        if not tagged and need_tag and need_tag.lower() == "true":
            git.tag(config.get(section, "tag_format") % newver)
            print("Tagged %s" % newver)
            tagged = True



import subprocess


def run(*args):
    return subprocess.run(["git"] + list(args), stdout=subprocess.PIPE)


def has_stage():
    return run("diff", "--cached", "--exit-code").returncode == 1


def add(filepath):
    return run("add", filepath)


def commit(message):
    return run("commit", "-m", message)


def tag(name):
    return run("tag", name)



import re


def bump(content: str, pattern: str, part: int) -> tuple:
    current_version = None
    new_version = None

    def replacer(m):
        nonlocal current_version, new_version

        version = m.group(1)
        if not version:
            raise ValueError("The regex pattern has no group")

        full_span = m.span(0)
        span = m.span(1)
        current_version = version
        version = list(map(int, version.split(".")))
        for k in range(len(version)):
            if k == part:
                version[k] += 1

            elif k > part:
                version[k] = 0

        new_version = ".".join(map(str, version))
        return (
            content[full_span[0] : span[0]]
            + new_version
            + content[span[1] : full_span[1]]
        )

    new_content = re.sub(pattern, replacer, content)

    if new_content == content:
        raise ValueError("Cannot find version string")

    return new_content, current_version, new_version



# flake8:noqa

V_MAJOR = "major"
V_MINOR = "minor"
V_PATCH = "patch"
V_BUILD = "build"
version_parts = (V_MAJOR, V_MINOR, V_PATCH, V_BUILD)

presets = {
    "file:VERSION": {
        "pattern": r"""(\d+.\d+.\d+(?:.\d+)?)""",
    },
    "file:package.json": {
        "pattern": r"""(\d+.\d+.\d+(?:.\d+)?)""",
    },
    "file:*/__init__.py": {
        "pattern": r"""__version__\s*=\s*(?:"|')(\d+.\d+.\d+(?:.\d+)?)(?:"|')"""
    },
}
commit_format = "v%s"
tag_format = "v%s"



from setuptools import setup


def read_version(module_name):
    from re import match, S
    from os.path import join, dirname

    f = open(join(dirname(__file__), module_name, "__init__.py"))
    return match(r".*__version__ = (\"|')(.*?)('|\")", f.read(), S).group(2)


setup(
    name="verbump",
    version=read_version("verbump"),
    description="Easy version incrementing CLI tool",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="MeyT",
    url="http://github.com/meyt/verbump",
    license="MIT",
    keywords="version versioning semver",
    python_requires=">=3.5",
    packages=["verbump"],
    entry_points={"console_scripts": ["verbump = verbump.cli:main"]},
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Unix",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Libraries",
    ],
)
